import { useState, useEffect, useCallback, useContext, useMemo } from "react";
import { Stack, LinearProgress, Box, Alert } from "@mui/material";
import { LogStreamConsole } from "../LogStream/LogStreamConsole";
import { LogStreamConsoleDialog } from "../LogStream/LogStreamConsoleDialog";
import { TimeRange } from "../Inputs/TimeRange";
import { PopoutButton } from "../Buttons/PopoutButton";
import { useCustomSnackbar } from "../snackbar/Snackbar";
import { closeSnackbar } from "notistack";
import { apiContext } from "../../../api/DeployApi";
import {
  useAPIMethod,
  usePolling,
  formatDateAndTime
} from "@ess-ics/ce-ui-common";
import Convert from "ansi-to-html";
import { getErrorMessage, isAbortError } from "../Helper";

const TIME_RANGE_VALUES = [
  {
    value: 720,
    text: "12 hours"
  },
  {
    value: 10080,
    text: "7 days"
  }
];

/**
 * hostName: name of the host (mandatory) which will be the qery parameter for backend call
 * iocName: name of ioc (optional). Used when querying procServ log
 * isSyslog: boolean, if true -> SysLog will be queried, if false -> procServLog will be queried
 */

const LOG_POLL_INTERVAL = 5000;
export function LokiPanel({ hostName, iocName, isSyslog }) {
  const showWarning = useCustomSnackbar();
  const client = useContext(apiContext);

  const handleError = useCallback(
    (error) => {
      if (error && !isAbortError(error)) {
        showWarning(getErrorMessage(error));
      }
    },
    [showWarning]
  );

  const [timeRange, setTimeRange] = useState(720);
  const [logDialogOpen, setLogDialogOpen] = useState(false);
  const [periodChange, setPeriodChange] = useState(false);
  const [alertIds, setAlertIds] = useState([]);
  const [html, setHtml] = useState("");

  const preprocessLog = useCallback(
    (logData, showWarning, timeRange, alertIds, setAlertIds) => {
      if (!logData) {
        return "";
      }

      if (logData?.warning && alertIds?.length === 0) {
        const warningId = showWarning(logData.warning, "warning");
        setAlertIds((alertIds) => [...alertIds, warningId]);
      }

      if (logData && logData.lines?.length > 0) {
        const logHtml = logData.lines.map((line) => formatLogLine(line));
        return `<html>
                  <head><meta http-equiv="Content-Type" content="text/html; charset=utf-8">
                    <style type="text/css">.logdate { color: #5cb85c; } body.pre{font-family: Monaco, Menlo, Consolas, "Courier New", monospace;}
                    </style>
                  </head>
                  <body>
                    <pre>${logHtml.join("")}</pre>
                  </body>
                </html>`;
      }

      return `<html><body><pre> - No messages found for ${TIME_RANGE_VALUES.find((range) => range.value === timeRange).text} period - </pre></body></html>`;
    },
    []
  );

  const logsToPreprocess = useCallback(
    (
      isSysLog,
      logData,
      procServLog,
      showWarning,
      timeRange,
      alertIds,
      setAlertIds
    ) => {
      if (isSysLog === true) {
        return preprocessLog(
          logData,
          showWarning,
          timeRange,
          alertIds,
          setAlertIds
        );
      }

      return preprocessLog(
        procServLog,
        showWarning,
        timeRange,
        alertIds,
        setAlertIds
      );
    },
    [preprocessLog]
  );

  const params = useMemo(
    () => ({
      host_name: hostName,
      ioc_name: iocName,
      time_range: timeRange
    }),
    [hostName, iocName, timeRange]
  );

  const {
    value: sysLogData,
    wrapper: getSysLogData,
    loading: sysLoading,
    error: sysLogError,
    abort: getSysLogDataAbort
  } = useAPIMethod({
    fcn: client.apis.Hosts.fetchSyslogLines,
    call: false,
    params
  });

  const {
    value: procServLog,
    wrapper: getProcServLog,
    loading: procServLoading,
    error: procServLogError,
    abort: getProcServLogAbort
  } = useAPIMethod({
    fcn: client.apis.IOCs.fetchProcServLogLines,
    call: false,
    params
  });

  // show sysLogErrors on snackBar
  useEffect(() => {
    handleError(sysLogError);
  }, [sysLogError, handleError]);

  // show procServErrors on snackBar
  useEffect(() => {
    handleError(procServLogError);
  }, [procServLogError, handleError]);

  const handleTimeRangeChange = (event) => {
    setPeriodChange(true);
    setTimeRange(event.target?.value);
  };

  // remove progressBar if intervall has been changed, and data received
  useEffect(() => {
    setPeriodChange(false);
  }, [sysLogData, procServLog]);

  // remove the logline limit exceed warning when user leaves component
  useEffect(() => {
    return () => {
      alertIds.forEach((id) => {
        closeSnackbar(id);
      });
    };
  }, [alertIds]);

  const updateLogs = useCallback(() => {
    if (isSyslog === true) {
      getSysLogData(hostName, timeRange);
    } else {
      getProcServLog(hostName, iocName, timeRange);
    }
  }, [getSysLogData, getProcServLog, hostName, iocName, isSyslog, timeRange]);

  const hasLogError = !!sysLogError || !!procServLogError;
  const hasLogData = !!sysLogData || !!procServLog;
  const hasAbortError =
    isAbortError(sysLogError) || isAbortError(procServLogError);
  const logError = sysLogError || procServLogError;

  usePolling(
    updateLogs,
    sysLoading || procServLoading || hasLogError,
    LOG_POLL_INTERVAL,
    isSyslog ? getSysLogDataAbort : getProcServLogAbort
  );

  useEffect(() => {
    setHtml(
      logsToPreprocess(
        isSyslog,
        sysLogData,
        procServLog,
        showWarning,
        timeRange,
        alertIds,
        setAlertIds
      )
    );
  }, [
    setHtml,
    alertIds,
    isSyslog,
    logsToPreprocess,
    procServLog,
    showWarning,
    sysLogData,
    timeRange
  ]);

  const dataReady = () => {
    return sysLogData || procServLog;
  };

  if (hasLogError && !hasAbortError) {
    return <Alert severity="error">{getErrorMessage(logError)}</Alert>;
  }

  if (!hasLogData && !hasLogError) {
    return (
      <div style={{ width: "100%" }}>
        <LinearProgress color="primary" />
      </div>
    );
  }

  return (
    <Box>
      <Stack
        display="flex"
        flexDirection="row"
        justifyContent="space-between"
        sx={{ marginBottom: (theme) => theme.spacing(2) }}
      >
        <TimeRange
          selected={timeRange}
          values={TIME_RANGE_VALUES}
          onTimeRangeChange={handleTimeRangeChange}
        />
        <PopoutButton
          title="Popout log"
          onButtonClick={() => setLogDialogOpen(true)}
        />
      </Stack>
      <LogStreamConsoleDialog
        title={isSyslog ? "Host log stream" : "IOC log stream"}
        log={html}
        open={logDialogOpen}
        dataReady={dataReady}
        loading={periodChange}
        onDialogClose={() => setLogDialogOpen(false)}
      >
        <Box sx={{ marginBottom: (theme) => theme.spacing(2) }}>
          <TimeRange
            selected={timeRange}
            values={TIME_RANGE_VALUES}
            onTimeRangeChange={handleTimeRangeChange}
          />
        </Box>
      </LogStreamConsoleDialog>
      <LogStreamConsole
        log={html}
        dataReady={dataReady}
        height="500px"
        loading={periodChange}
      />
    </Box>
  );
}

function formatLogLine(logLine) {
  const convert = new Convert();

  return (
    "<span><span class=logdate>" +
    formatDateAndTime(logLine.logDate) +
    "</span> " +
    convert.toHtml(logLine.logMessage) +
    "<br/></span>"
  );
}
