import {
  useState,
  useCallback,
  useRef,
  useEffect,
  useContext,
  useMemo
} from "react";
import { apiContext } from "../../api/DeployApi";
import { useAPIMethod, usePolling } from "@ess-ics/ce-ui-common";
import { LogStreamConsole } from "../common/LogStream/LogStreamConsole";
import { LogStreamConsoleDialog } from "../common/LogStream/LogStreamConsoleDialog";
import { PopoutButton } from "../common/Buttons/PopoutButton";
import { Alert, LinearProgress, Stack } from "@mui/material";
import { getErrorMessage, isAbortError } from "../common/Helper";
import useCustomSnackbar from "../common/snackbar";

const LOG_POLL_INTERVAL = 5000;

export function DeploymentJobOutput({ job }) {
  const [consoleDialogOpen, setConsoleDialogOpen] = useState(false);
  const client = useContext(apiContext);
  const finalResultsNeeded = useRef(true);
  const params = useMemo(
    () => ({
      awx_job_id: job.awxJobId
    }),
    [job]
  );
  const showWarning = useCustomSnackbar();

  const {
    value: log,
    wrapper: getLogById,
    loading: logLoading,
    dataReady: logDataReady,
    error: logError,
    abort: abortGetLogById
  } = useAPIMethod({
    fcn: client.apis.Jobs.fetchDeploymentJobLog,
    params
  });

  const getLog = useCallback(() => {
    if (!job.finishedAt || finalResultsNeeded.current) {
      getLogById(job.jobId);
      finalResultsNeeded.current = !job.finishedAt;
    }
  }, [job.finishedAt, job.jobId, getLogById]);

  const hasAbortError = isAbortError(logError);
  const showLoading = !log || !job.startTime;

  usePolling(
    getLog,
    logLoading || !logDataReady || logError,
    LOG_POLL_INTERVAL,
    abortGetLogById
  );

  useEffect(() => {
    if (logError && !hasAbortError) {
      showWarning(getErrorMessage(logError));
    }
  }, [logError, showWarning]);

  useEffect(() => {
    finalResultsNeeded.current = true;
  }, [job.jobId]);

  if (logError && !hasAbortError) {
    return (
      <Stack>
        <Alert severity="error">{getErrorMessage(logError)}</Alert>
      </Stack>
    );
  }

  if (showLoading) {
    return (
      <Stack>
        <LinearProgress color="primary" />
      </Stack>
    );
  }

  return (
    <Stack>
      <Stack
        display="flex"
        flexDirection="row"
        justifyContent="flex-end"
        sx={{ marginBottom: (theme) => theme.spacing(2) }}
      >
        <PopoutButton
          title="Popout log"
          onButtonClick={() => setConsoleDialogOpen(true)}
        />
      </Stack>
      <LogStreamConsoleDialog
        title="Job log stream"
        loading={showLoading}
        open={consoleDialogOpen}
        log={log?.stdoutHtml}
        dataReady={logDataReady}
        onDialogClose={() => setConsoleDialogOpen(false)}
      />
      <LogStreamConsole
        log={log.stdoutHtml}
        dataReady={logDataReady}
        height="500px"
      />
    </Stack>
  );
}
